# principles and practice of sem, 5th ed.
# rex b. kline, guilford press, 2023

# chapter 10, table 10.1, analysis 2
# conduct power and precision analyses for the
# roth et al. (1989) parametric path model of illness:

# 1. maccallum-rmsea for whole model (semTools)
# 2. monte carlo for individual parameters (WebPower)
# 3. kelley-lai precision (MBESS)

# to avoid the problem that some R packages share the same name 
# for different functions, all functions are specified next as 

# package::function

# which prevents masking, or the default hiding of 
# # a function with a redundant name from a package used next

date()

v <- R.Version()
print(paste0(v$language, " version ", v$major, ".",
 v$minor, " (", v$year, "-", v$month, "-", v$day, ")"))

library(lavaan)
library(semTools)
library(WebPower)
library(MBESS)

# get citation information
citation("semTools", auto = TRUE)
citation("lavaan", auto = TRUE)
citation("WebPower", auto = TRUE)
citation("MBESS", auto = TRUE)

# 1a. maccallum-rmsea for whole model
# power at N = 373

# exact fit
semTools::findRMSEApower(0, .05, 5, 373, .05, 1)
# close fit
semTools::findRMSEApower(.05, .08, 5, 373, .05, 1)
# not-close fit
semTools::findRMSEApower(.05, .01, 5, 373, .05, 1)
# poor fit
semTools::findRMSEApower(.10, .05, 5, 373, .05, 1)

# 1b. maccallum-rmsea for whole model
# minimum N for power at least .90

# exact fit
semTools::findRMSEAsamplesize(0, .05, 5, .90, .05, 1)
# close fit
semTools::findRMSEAsamplesize(.05, .08, 5, .90, .05, 1)
# not-close fit
semTools::findRMSEAsamplesize(.05, .01, 5, .90, .05, 1)
# poor fit
semTools::findRMSEAsamplesize(.10, .05, 5, .90, .05, 1)

# 2. monte carlo for individual parameters
# specify model with fixed parameter values
# results may vary slightly if rerun due to
# simulated random sampling

roth.model <- '
# direct effects
 fitness ~ a*exercise + start(.05)*exercise
 stress ~ c*hardy + start(-.18)*hardy
 illness ~ b*fitness + start(-.35)*fitness + d*stress + start(.38)*stress
# variances and covariances for exogenous variables
 exercise ~~ start(4500.00)*exercise
 hardy ~~ start(1400.00)*hardy
 exercise ~~ start(251.00)*hardy
 fitness ~~ start(304.00)*fitness
 stress ~~ start(1045.00)*stress
 illness ~~ start(3600.00)*illness '

roth.indirect <- '
 ab := a*b
 cd := c*d '

roth.model.ml <- WebPower::wp.mc.sem.basic(model = roth.model,
 indirect = roth.indirect, nobs = 373, nrep = 1000)
lavaan::summary(roth.model.ml)

# 3. kelley-lai precision 
MBESS::ss.aipe.rmsea(RMSEA = .04, df = 5, width = .035, conf.level = .95)

