# principles and practice of sem, 5th ed.
# rex b. kline, guilford press, 2023

# chapter 11, table 11.2, analysis 2
# fit equivalent versions of the romney et al. (1992)
# conventional medical model to the same data

# to avoid the problem that some R packages share the same name 
# for different functions, all functions are specified next as 

# package::function

# which prevents masking, or the default hiding of 
# a function with a redundant name from a package used next

date()

v <- R.Version()
print(paste0(v$language, " version ", v$major, ".",
 v$minor, " (", v$year, "-", v$month, "-", v$day, ")"))

library(lavaan)

# get citation information
citation("lavaan", auto = TRUE)

# input the correlations in lower diagnonal form
romneyLower.cor <- '
 1.00
  .53 1.00
  .15  .18 1.00
  .52  .29 -.05 1.00
  .30  .34  .23  .09 1.00 '

# name the variables and convert to full correlation matrix
romney.cor <- lavaan::getCov(romneyLower.cor, names = c("morale", "symptoms",
 "dysfunction", "relations", "ses"))

# display the correlations
romney.cor

# add the standard deviations and convert to covariances
romney.cov <- lavaan::cor2cov(romney.cor, sds = c(3.75,17.00,19.50,
 3.50,24.70))

# display the covariances
romney.cov

# specify fit statistics for abbreviated output
fit.stats = c("chisq", "df", "pvalue", "cfi", "srmr",
 "rmsea", "rmsea.ci.lower", "rmsea.ci.upper")

# specify original and 3 equivalent models

# original conventional medical model
# figure 11.2(a)

original.model <- '
 ses ~ symptoms + dysfunction
 morale ~ symptoms + ses
 relations ~ dysfunction + morale '

# equivalent model 1
# figure 11.2(b)

equivalent1.model <- '
 ses ~ symptoms + dysfunction
 morale ~ symptoms + ses
 relations ~ dysfunction + morale
 dysfunction ~ symptoms '

# equivalent model 2
# figure 11.3(c)

equivalent2.model <- '
 ses ~ symptoms
 morale ~ symptoms + ses
 relations ~ dysfunction + morale
 dysfunction ~ symptoms + ses '

# equivalent model 3
# figure 11.3(d)

equivalent3.model <- '
 ses ~ symptoms + morale
 morale ~ symptoms
 relations ~ dysfunction + morale
 dysfunction ~ symptoms + ses '

# fit models to data 

original <- lavaan::sem(original.model, sample.cov = romney.cov,
 sample.nobs = 469, fixed.x = FALSE, sample.cov.rescale = FALSE)

equivalent1 <- lavaan::sem(equivalent1.model, sample.cov = romney.cov,
 sample.nobs = 469, fixed.x = FALSE, sample.cov.rescale = FALSE)

equivalent2 <- lavaan::sem(equivalent2.model, sample.cov = romney.cov,
 sample.nobs = 469, fixed.x = FALSE, sample.cov.rescale = FALSE)

equivalent3 <- lavaan::sem(equivalent3.model, sample.cov = romney.cov,
 sample.nobs = 469, fixed.x = FALSE, sample.cov.rescale = FALSE)

# global fit statistics
# same for all models

lavaan::fitMeasures(original, fit.stats)
lavaan::fitMeasures(equivalent1, fit.stats)
lavaan::fitMeasures(equivalent2, fit.stats)
lavaan::fitMeasures(equivalent3, fit.stats)

# residuals
# same for all models for each of residual

# raw residuals
lavaan::residuals(original, type = "raw")
lavaan::residuals(equivalent1, type = "raw")
lavaan::residuals(equivalent2, type = "raw")
lavaan::residuals(equivalent3, type = "raw")

# standardized residuals
lavaan::residuals(original, type = "standardized.mplus")
lavaan::residuals(equivalent1, type = "standardized.mplus")
lavaan::residuals(equivalent2, type = "standardized.mplus")
lavaan::residuals(equivalent3, type = "standardized.mplus")

# correlation residuals
lavaan::residuals(original, type = "cor.bollen")
lavaan::residuals(equivalent1, type = "cor.bollen")
lavaan::residuals(equivalent2, type = "cor.bollen")
lavaan::residuals(equivalent3, type = "cor.bollen")

# parameter estimates
# these results vary over models

lavaan::summary(original, fit.measures = FALSE, header = FALSE,
 standardized = TRUE)
lavaan::summary(equivalent1, fit.measures = FALSE, header = FALSE, 
 standardized = TRUE)
lavaan::summary(equivalent2, fit.measures = FALSE, header = FALSE,
 standardized = TRUE)
lavaan::summary(equivalent3, fit.measures = FALSE, header = FALSE,
 standardized = TRUE)
