# principles and practice of sem, 5th ed.
# rex b. kline, guilford press, 2023

# chapter 12, table 12.2, analysis 2
# multiple group analysis of a recursive path model of
# achievement and delinquency by lynam et al. (1993)
# covariance structure and mean structure

# to avoid the problem that some R packages share the same name 
# for different functions, all functions are specified next as 

# package::function

# which prevents masking, or the default hiding of 
# a function with a redundant name from a package used next

date()

v <- R.Version()
print(paste0(v$language, " version ", v$major, ".",
 v$minor, " (", v$year, "-", v$month, "-", v$day, ")"))

library(lavaan)
options(width = 130)

# get citation information
citation("lavaan", auto = TRUE)

# black, n = 214
# white, n = 181
# input the correlations in lower diagnonal form

blackLower.cor <- '
 1.00
  .08 1.00
  .28  .30 1.00
  .05  .21  .50 1.00
 -.11 -.17 -.26 -.33 1.00 '

whiteLower.cor <- '
 1.00
  .25 1.00
  .37  .40 1.00
  .27  .28  .61 1.00
 -.11 -.20 -.31 -.21 1.00 '

# name the variables and convert to full correlation matrix
black.cor <- lavaan::getCov(blackLower.cor, names = c("ses", "effort", "viq",
 "achieve", "delinq"))
white.cor <- lavaan::getCov(whiteLower.cor, names = c("ses", "effort", "viq",
 "achieve", "delinq"))

# display the correlations
black.cor
white.cor

# add the standard deviations and convert to covariances
black.cov <- lavaan::cor2cov(black.cor, sds = c(10.58,1.35,13.62,.79,1.63))
white.cov <- lavaan::cor2cov(white.cor, sds = c(11.53,1.32,16.32,.96,1.45))

# input group means
black.mean <- c(31.96,-.01,93.76,2.51,1.40)
white.mean <- c(34.64,.05,104.18,2.88,1.22)

# display the covariances and means
black.cov
black.mean
white.cov
white.mean

# specify fit statistics for abbreviated output
fit.stats = c("chisq", "df", "pvalue", "cfi", "srmr",
 "rmsea", "rmsea.ci.lower", "rmsea.ci.upper")

# combine covariances matrices, mean vectors, and group sizes
# into single list objects

combined.cov <- list(black = black.cov, white = white.cov)
combined.mean <- list(black = black.mean, white = white.mean)
combined.n <- list(black = 214, white = 181)

# model 3 with covariance and mean structures explicitly
# specified in syntax
# parameter names impose equality constraints or allow
# free estimation in each group
# individual direct effects explicitly defined in each group

lynam3.model <- '
 achieve ~ c(a1,a1)*ses + c(a2,a2)*effort + c(a3,a3)*viq
 delinq ~ c(b4,b5)*achieve + c(b1,b1)*ses + c(b2,b2)*effort + c(b3,b3)*viq
 ses ~~ ses
 effort ~~ effort
 viq ~~ viq
 ses ~~ effort
 ses ~~ viq
 effort ~~ viq
 ses ~ 1
 effort ~ 1 
 viq ~ 1
 achieve ~ c(d1,d1)*1
 delinq ~ 1 
# indirect effects on delinquency through achievement
# defined next by causal variable
# e.g., ses1 is ses -> achieve -> delinq for group 1
 ses1 := a1*b4
 ses2 := a1*b5
 effort1 := a2*b4
 effort2 := a2*b5
 viq1 := a3*b4
 viq2 := a3*b5 '

lynam3 <- lavaan::sem(lynam3.model, sample.cov = combined.cov,
 sample.mean = combined.mean, sample.nobs = combined.n,
 fixed.x = FALSE)

# fit statistics, parameter estimates, and residuals are the same
# for model 3 in analysis 1, they are reproduced here
# for convenience
lavaan::fitMeasures(lynam3, fit.stats)
lavaan::summary(lynam3, standardized = TRUE, fit.measures = FALSE, 
 header = FALSE)
lavaan::residuals(lynam3, type = "raw")
lavaan::residuals(lynam3, type = "standardized.mplus")
lavaan::residuals(lynam3, type = "normalized")
lavaan::residuals(lynam3, type = "cor.bollen")





