# principles and practice of sem, 5th ed.
# rex b. kline, guilford press, 2023

# chapter 12, table 12.2, analysis 1
# multiple group analysis of a recursive path model of
# achievement and delinquency by lynam et al. (1993)
# covariance structure and mean structure

# to avoid the problem that some R packages share the same name 
# for different functions, all functions are specified next as 

# package::function

# which prevents masking, or the default hiding of 
# a function with a redundant name from a package used next

date()

v <- R.Version()
print(paste0(v$language, " version ", v$major, ".",
 v$minor, " (", v$year, "-", v$month, "-", v$day, ")"))

library(lavaan)
options(width = 130)

# get citation information
citation("lavaan", auto = TRUE)

# black, n = 214
# white, n = 181
# input the correlations in lower diagnonal form

blackLower.cor <- '
 1.00
  .08 1.00
  .28  .30 1.00
  .05  .21  .50 1.00
 -.11 -.17 -.26 -.33 1.00 '

whiteLower.cor <- '
 1.00
  .25 1.00
  .37  .40 1.00
  .27  .28  .61 1.00
 -.11 -.20 -.31 -.21 1.00 '

# name the variables and convert to full correlation matrix
black.cor <- lavaan::getCov(blackLower.cor, names = c("ses", "effort", "viq",
 "achieve", "delinq"))
white.cor <- lavaan::getCov(whiteLower.cor, names = c("ses", "effort", "viq",
 "achieve", "delinq"))

# display the correlations
black.cor
white.cor

# add the standard deviations and convert to covariances
black.cov <- lavaan::cor2cov(black.cor, sds = c(10.58,1.35,13.62,.79,1.63))
white.cov <- lavaan::cor2cov(white.cor, sds = c(11.53,1.32,16.32,.96,1.45))

# input group means
black.mean <- c(31.96,-.01,93.76,2.51,1.40)
white.mean <- c(34.64,.05,104.18,2.88,1.22)

# display the covariances and means
black.cov
black.mean
white.cov
white.mean

# specify fit statistics for abbreviated output
fit.stats = c("chisq", "df", "pvalue", 
 "rmsea", "rmsea.ci.lower", "rmsea.ci.upper", "cfi", "srmr")

# combine covariances matrices, mean vectors, and group sizes
# into single list objects

combined.cov <- list(black = black.cov, white = white.cov)
combined.mean <- list(black = black.mean, white = white.mean)
combined.n <- list(black = 214, white = 181)

# specify basic covariance model

lynam.model <- '
 achieve ~ ses + effort + viq
 delinq ~ achieve + ses + effort + viq '

# mean structure and equality constraints for direct
# effects or intercepts are specified in syntax for 
# fitting each model to the data

# model 1
# all 7 direct effect constrained to equality
# over groups

lynam1 <- lavaan::sem(lynam.model, sample.cov = combined.cov,
 sample.mean = combined.mean, sample.nobs = combined.n,
 group.equal = c("regressions"), fixed.x = FALSE, meanstructure = TRUE)

# model 2
# direct effects constrained to equality over groups
# but the constraint on achievement -> delinguency is 
# released in syntax for "group.partial"
# so 6 direct effects are constrained over groups

lynam2 <- lavaan::sem(lynam.model, sample.cov = combined.cov,
 sample.mean = combined.mean, sample.nobs = combined.n,
 group.equal = c("regressions"), group.partial = c("delinq ~ achieve"),
 fixed.x = FALSE, meanstructure = TRUE)

# model 3
# retained model
# 6 direct effects constrained to equality over groups
# achievement -> delinguency is a free parameter in both groups
# intercept for achievement constrained to equality

lynam3 <- lavaan::sem(lynam.model, sample.cov = combined.cov,
 sample.mean = combined.mean, sample.nobs = combined.n,
 group.equal = c("regressions", "intercepts"),
 group.partial = c("delinq ~ achieve", "delinq ~ 1"),
 fixed.x = FALSE, meanstructure = TRUE)

# model 4
# 6 direct effects constrained to equality over groups
# achievement -> delinguency is a free parameter in both groups
# intercept for deleinquency constrained to equality

lynam4 <- lavaan::sem(lynam.model, sample.cov = combined.cov,
 sample.mean = combined.mean, sample.nobs = combined.n,
 group.equal = c("regressions", "intercepts"),
 group.partial = c("delinq ~ achieve", "achieve ~ 1"),
 fixed.x = FALSE, meanstructure = TRUE)

# global fit statistics
lavaan::fitMeasures(lynam1, fit.stats)
lavaan::fitMeasures(lynam2, fit.stats)
lavaan::fitMeasures(lynam3, fit.stats)
lavaan::fitMeasures(lynam4, fit.stats)

# chi square difference tests
lavaan::anova(lynam1, lynam2)
lavaan::anova(lynam2, lynam3)
lavaan::anova(lynam2, lynam4)

# model 1
# parameter estimates, residuals
# all mean residuals are zero

lavaan::summary(lynam1, standardized = TRUE, fit.measures = FALSE)
lavaan::fitted(lynam1)
lavaan::residuals(lynam1, type = "raw")
lavaan::residuals(lynam1, type = "standardized.mplus")
lavaan::residuals(lynam1, type = "normalized")
lavaan::residuals(lynam1, type = "cor.bollen")

# model 2
# parameter estimates, residuals
# all mean residuals are zero

lavaan::summary(lynam2, standardized = TRUE, fit.measures = FALSE)
lavaan::fitted(lynam2)
lavaan::residuals(lynam2, type = "raw")
lavaan::residuals(lynam2, type = "standardized.mplus")
lavaan::residuals(lynam2, type = "normalized")
lavaan::residuals(lynam2, type = "cor.bollen")

# model 3
# retained model
# parameter estimates, residuals, predicted means
# not all mean residuals are zero

lavaan::summary(lynam3, standardized = TRUE, fit.measures = FALSE)
lavaan::fitted(lynam3)
lavaan::residuals(lynam3, type = "raw")
lavaan::residuals(lynam3, type = "standardized.mplus")
lavaan::residuals(lynam3, type = "normalized")
lavaan::residuals(lynam3, type = "cor.bollen")

# model 4
# parameter estimates, residuals, predicted means
# not all mean residuals are zero

lavaan::summary(lynam4, standardized = TRUE, fit.measures = FALSE)
lavaan::fitted(lynam4)
lavaan::residuals(lynam4, type = "raw")
lavaan::residuals(lynam4, type = "standardized.mplus")
lavaan::residuals(lynam4, type = "normalized")
lavaan::residuals(lynam4, type = "cor.bollen")
