# principles and practice of sem, 5th ed.
# rex b. kline, guilford press, 2023

# chapter 14, table 14.1, analysis 2
# two-factor cfa model for the kabc-1 analyzed
# in the 10-year-old normative sample for the test

# to avoid the problem that some R packages share the same name 
# for different functions, all functions are specified next as 

# package::function

# which prevents masking, or the default hiding of 
# a function with a redundant name from a package used next

date()

v <- R.Version()
print(paste0(v$language, " version ", v$major, ".",
 v$minor, " (", v$year, "-", v$month, "-", v$day, ")"))

library(lavaan)
library(semTools)

# get citation information
citation("lavaan", auto = TRUE)
citation("semTools", auto = TRUE)

# maccallum-rmsea for whole model
# exact fit test
# power at N = 200
semTools::findRMSEApower(0, .05, 19, 200, .05, 1)

# minimum N for power at least .90
semTools::findRMSEAsamplesize(0, .05, 19, .90, .05, 1)

# input the correlations in lower diagnonal form
kabcLower.cor <- '
 1.00
 .39 1.00
 .35  .67 1.00
 .21  .11  .16 1.00
 .32  .27  .29  .38 1.00
 .40  .29  .28  .30  .47 1.00
 .39  .32  .30  .31  .42  .41 1.00
 .39  .29  .37  .42  .58  .51  .42 1.00 '

# name the variables and convert to full correlation matrix
# hm, hand movements; nr, number recall; wo, word order; gc, gestalt closure; 
# tr, triangles; sm, spatial memory; ma, matrix analogies; ps, photo series
kabc.cor <- lavaan::getCov(kabcLower.cor, names = c("hm", "nr", "wo",
 "gc", "tr", "sm", "ma", "ps"))
# display correlations
kabc.cor

# add the standard deviations and convert to covariances
kabc.cov <- lavaan::cor2cov(kabc.cor, sds = c(3.40,2.40,2.90,2.70,2.70,4.20,2.80,3.00))

# display covariances
kabc.cov

# specify 2-factor cfa model
# factors are sequential, simultaneous
# indicators hm and gc automatically
# specified as reference variables for this
# respective factors
# factor covariance automatically specified
 
kabc2.model <- '
Sequent =~ hm + nr + wo
Simultan =~ gc + tr + sm + ma + ps '

# fit model to data
# variances calculated with N in the denominator, not N - 1

kabc2 <- lavaan::sem(kabc2.model, sample.cov = kabc.cov, sample.nobs = 200)
summary(kabc2, fit.measures = TRUE, standardized = TRUE, rsquare = TRUE)

# unstandardized estimates with standard errors
lavaan::parameterEstimates(kabc2)

# standardized estimates with standard errors
lavaan::standardizedSolution(kabc2)

# implied covariance matrix
lavaan::fitted(kabc2)

# implied correlation matrix for all variables (observed, latent)
lavaan::lavInspect(kabc2, add.labels = TRUE, "cor.all")

# residuals
lavaan::residuals(kabc2, type = "raw")
lavaan::residuals(kabc2, type = "standardized.mplus")
lavaan::residuals(kabc2, type = "cor.bollen")

# mi values
lavaan::modindices(kabc2)

# calculate factor reliability coefficients
# note that semTools calculates AVE based on the unstandardized
# solution, not the standardized solution, so the result for
# AVE below will not match those in chapter 14, which
# are based on the standardized solution

semTools::reliability(kabc2)


