# principles and practice of sem, 5th ed.
# rex b. kline, guilford press, 2023

# chapter 16, table 16.2, analysis 1
# model of stress and depression as a function of
# acculturation and ses analyzed as a partial sr
# model that assumes reflective measurement

# to avoid the problem that some R packages share the same name 
# for different functions, all functions are specified next as 

# package::function

# which prevents masking, or the default hiding of 
# a function with a redundant name from a package used next

date()

v <- R.Version()
print(paste0(v$language, " version ", v$major, ".",
 v$minor, " (", v$year, "-", v$month, "-", v$day, ")"))

library(lavaan)
library(semTools)

# get citation information
citation("lavaan", auto = TRUE)
citation("semTools", auto = TRUE)

# input the correlations in lower diagnonal form
shenLower.cor <- '
1.00
 .44 1.00
 .69  .54 1.00
 .21  .08  .16 1.00
 .23  .15  .19  .19 1.00
 .12  .08  .08  .08 -.03 1.00
 .09  .06  .04  .01 -.02  .38 1.00
 .03  .02 -.02 -.07 -.11  .37  .46 1.00 '

# name the variables and convert to full correlation matrix
shen.cor <- lavaan::getCov(shenLower.cor, names = c("acculscl", "status",
 "percent", "educ", "income", "interpers", "job", "scl90d"))

# add the standard deviations and convert to covariances
shen.cov <- lavaan::cor2cov(shen.cor, sds = c(3.60,3.30,2.45,3.27,3.44,2.99,
 3.58,3.70))

# display correlations and covariances
shen.cor
shen.cov

# maccallum-rmsea for whole model
# exact fit test
# power at N = 983
semTools::findRMSEApower(0, .05, 16, 983, .05, 1)

# minimum N for power at least .90
semTools::findRMSEAsamplesize(0, .05, 16, .90, .05, 1)

# scl90d score reliability (alpha) is .90 from 
# derogatis et al. (1976)
# sample variance is 3.70**2 = 13.690
# error variance fixed to (1 - .90) * 13.690 = 1.369

# specify reflective model

shenSR.model <- '
# measurement model with error covariance
 Acculturation =~ acculscl + status + percent
 status ~~ percent
 SES =~ educ + income
 Stress =~ interpers + job
 Depression =~ 1*scl90d
 scl90d ~~ 1.369*scl90d
# structural model
 Stress ~ a*Acculturation
 Depression ~ SES + b*Stress 
# define indirect effect of acculturation
 ab := a * b '

# fit model to data
shenSR <- lavaan::sem(shenSR.model, sample.cov = shen.cov,
 sample.nobs = 983, fixed.x = FALSE)
lavaan::summary(shenSR, fit.measures = TRUE, rsquare = TRUE)
lavaan::standardizedSolution(shenSR)

# predicted covariances
lavaan::fitted(shenSR)

# predicted correlation matrix for indicators
lavaan::lavInspect(shenSR, "cor.ov")

# predicted covariance matrix for factors
lavaan::lavInspect(shenSR, "cov.lv")

# predicted correlation matrix for factors
lavaan::lavInspect(shenSR, "cor.lv")

# residuals
lavaan::residuals(shenSR, type = "raw")
lavaan::residuals(shenSR, type = "standardized.mplus")
lavaan::residuals(shenSR, type = "cor.bollen")

# calculate factor reliability coefficients (semTools)
# note that semTools calculates AVE based on the unstandardized
# solution, not the standardized solution, so the result for
# AVE below will not match those in the composite models chapter,
# which are based on the standardized solution

semTools::reliability(shenSR)
