# principles and practice of sem, 5th ed.
# rex b. kline, guilford press, 2023

# chapter 17, table 17.1, analysis 2
# 2sls estimation with model-implied instruments
# in a small-sample cfa (n = 103)

# to avoid the problem that some R packages share the same name 
# for different functions, all functions are specified next as 

# package::function

# which prevents masking, or the default hiding of 
# a function with a redundant name from a package used next

date()

v <- R.Version()
print(paste0(v$language, " version ", v$major, ".",
 v$minor, " (", v$year, "-", v$month, "-", v$day, ")"))

library(lavaan)
library(MIIVsem)

# get citation information
citation("lavaan", auto = TRUE)
citation("MIIVsem", auto = TRUE)

# input the correlations in lower diagnonal form
sabatelliLower.cor <- '
 1.000
  .740 1.000
  .265  .422 1.000
  .305  .401  .791 1.000
  .315  .351  .662  .587 1.000 '

# name the variables and convert to full correlation matrix
sabatelli.cor <- lavaan::getCov(sabatelliLower.cor, names = c("problems",
 "intimacy", "father", "mother", "both"))

# add the standard deviations and convert to covariances
sabatelli.cov <- lavaan::cor2cov(sabatelli.cor, sds = c(32.936,22.749,
 13.390,13.679,14.382))

# display correlations and covariances

sabatelli.cor
sabatelli.cov

# specify model in lavaan syntax

sabatelli.model <- '
 Marital =~ problems + intimacy
 FOE =~ father + mother + both '

# generate list of model-implied instruments
# for each equation

MIIVsem::miivs(sabatelli.model)

# 2sls estimates with sargan test
# over multiple instruments
sabatelli <- MIIVsem::miive(sabatelli.model, sample.cov = sabatelli.cov,
 sample.nobs = 103, var.cov = TRUE)
lavaan::summary(sabatelli, rsquare = TRUE)

# generate residuals in lavaan
# all parameters are specified as fixed
# based on 2SLS estimates,
# next the model is fitted to sample data matrix
# the usual global fit statistics in ml estimation
# may have no meaningful interpretation

sabatelliFixed.model <- '
# common factors
 Marital =~ 1.0*problems + .805*intimacy
 FOE =~ 1.0*father + .899*mother + .787*both 
# factor variances, covariances
 FOE ~~ 158.501*FOE
 Marital ~~ 157.495*FOE
 Marital ~~ 702.393*Marital
# indicator error variances
 father ~~ 20.856*father
 mother ~~ 54.195*mother
 both ~~ 103.301*both
 problems ~~ 422.427*problems
 intimacy ~~ 50.781*intimacy '

# fit model with fixed parameters to data
sabatelliFixed <- lavaan::sem(sabatelliFixed.model, sample.cov = sabatelli.cov,
 sample.nobs = 103)

# standardized parameter "estimates" listed 
# next are fixed to nonzero constants, and
# standard errors are undefined
lavaan::parameterEstimates(sabatelliFixed)

# residuals
lavaan::residuals(sabatelliFixed, type = "raw")
lavaan::residuals(sabatelliFixed, type = "standardized.mplus")
lavaan::residuals(sabatelliFixed, type = "cor.bollen")

