# principles and practice of sem, 5th ed.
# rex b. kline, guilford press, 2023

# chapter 19, table 19.2, analysis 2
# computation of blocked-error-r-squared for
# partial sr model with a casual loop 

# to avoid the problem that some R packages share the same name 
# for different functions, all functions are specified next as 

# package::function

# which prevents masking, or the default hiding of 
# a function with a redundant name from a package used next

date()

v <- R.Version()
print(paste0(v$language, " version ", v$major, ".",
 v$minor, " (", v$year, "-", v$month, "-", v$day, ")"))

library("lavaan")
library("semTools")

# get citations
citation(package = "lavaan", auto = TRUE)
citation(package = "semTools", auto = TRUE)

# input correlation matrix and 
# standard deviations

robbinsLower.cor <- '
 1.000
  .933 1.000
  .767  .738 1.000
  .760  .737  .614  1.000
  .820  .760  .664   .867 1.000
  .454  .466  .372   .368  .375 1.000
  .469  .480  .380   .457  .351  .498 1.000 '

robbins.cor <- lavaan::getCov(robbinsLower.cor, names = c("rule", "legal",
 "corrupt", "bandwidth", "internet", "trust", "monarchy"))

robbins.cov <- lavaan::cor2cov(robbins.cor, sds = c(1.008,1.931,
 1.424,2.585,1.569,.138,.394))

# display covariance matrix
robbins.cov

# specify full model

robbinsFull.model <- '
# common factors
 InfoTech =~ bandwidth + internet
InstitQual =~ legal + corrupt
# structural model with causal loop,
# correlated disturbances, and instruments
 InstitQual ~ trust + InfoTech
 trust ~ InstitQual + monarchy
# correlated disturbances
 InstitQual ~~ trust
# covariances between exogenous variables
 monarchy ~~ InfoTech '

# fit model to data, extract the model chi-square,
# unstandardized parameter estimates, and the 
# model-implied covariance matrix

robbinsFull <- lavaan::sem(robbinsFull.model, sample.cov = robbins.cov,
 sample.nobs = 64)
lavaan::lavInspect(robbinsFull, add.labels = TRUE, "test")
lavaan::parameterEstimates(robbinsFull)
lavaan::lavInspect(robbinsFull, add.labels = TRUE, "cov.all")

# save predicted covariance matrix

matrixFull <- lavaan::lavInspect(robbinsFull, add.labels = TRUE, "cov.all")

# model-implied variance, trust
matrixFull[5,5]
# model-implied variance, Quality
matrixFull[8,8]

# blocked-error model, trust

# disturbance variance and covariance for
# trust fixed to equal zero
# all other parameters fixed to equal their
# values in the full model

robbinsBlockTrust.model <- '
# common factors and indicators
 InfoTech =~ 1*bandwidth + .628*internet
 bandwidth ~~ 1.065*bandwidth
 internet ~~ .250*internet
 InfoTech ~~ 5.513*InfoTech
 InstitQual =~ 1*legal + .624*corrupt
 legal ~~ .469*legal
 corrupt ~~ .750*corrupt
 InstitQual ~~ .920*InstitQual
# structural model with causal loop
 InstitQual ~ 5.772*trust + .523*InfoTech
 trust ~ .022*InstitQual + .123*monarchy
# blocked error for trust
 trust ~~ 0*trust
 InstitQual ~~ 0*trust 
# covariances between exogenous variables
 monarchy ~~ .153*monarchy
 monarchy ~~ .384*InfoTech '

robbinsBlockTrust <- lavaan::sem(robbinsBlockTrust.model,
 sample.cov = robbins.cov, sample.nobs = 64)

# all parameter estimates are fixed including the 
# disturbance variance and covariance for trust
lavaan::parameterEstimates(robbinsBlockTrust)

# implied covariance matrix for all variables (observed, latent)
lavaan::lavInspect(robbinsBlockTrust, add.labels = TRUE, "cov.all")

# save implied covariance matrix for BlockTrust 
# and compute blocked-error R-square 
matrixBlockTrust <- lavaan::lavInspect(robbinsBlockTrust,
 add.labels = TRUE, "cov.all")
beR2Trust = matrixBlockTrust[5,5]/matrixFull[5,5]
beR2Trust

# blocked-error model, Quality

# disturbance variance and covariance for
# Quality fixed to equal zero
# all other parameters fixed to equal their
# values in the full model

robbinsBlockQuality.model <- '
# common factors and indicators
 InfoTech =~ 1*bandwidth + .628*internet
 bandwidth ~~ 1.065*bandwidth
 internet ~~ .250*internet
 InfoTech ~~ 5.513*InfoTech
 InstitQual =~ 1*legal + .624*corrupt
 legal ~~ .469*legal
 corrupt ~~ .750*corrupt
# structural model with causal loop
 InstitQual ~ 5.772*trust + .523*InfoTech
 trust ~ .022*InstitQual + .123*monarchy
# blocked error for Quality
 trust ~~ .013*trust
 InstitQual ~~ 0*InstitQual
 InstitQual ~~ 0*trust 
# covariances between exogenous variables
 monarchy ~~ .153*monarchy
 monarchy ~~ .384*InfoTech '

robbinsBlockQuality <- lavaan::sem(robbinsBlockQuality.model,
 sample.cov = robbins.cov, sample.nobs = 64)
# all parameter estimates are fixed including the 
# disturbance variance and covariance for Quality
lavaan::parameterEstimates(robbinsBlockQuality)

# implied covariance matrix for all variables (observed, latent)
lavaan::lavInspect(robbinsBlockQuality, add.labels = TRUE, "cov.all")

# save implied covariance matrix for BlockQuality 
# and compute blocked-error R-square 

matrixBlockQuality <- lavaan::lavInspect(robbinsBlockQuality,
 add.labels = TRUE, "cov.all")
beR2Quality = matrixBlockQuality[8,8]/matrixFull[8,8]
beR2Quality

