# principles and practice of sem, 5th ed.
# rex b. kline, guilford press, 2023

# chapter 21, table 21.2, analysis 2
# growth model for predicting change in alcohol use
# over ages 13–16 years

# to avoid the problem that some R packages share the same name 
# for different functions, all functions are specified next as 

# package::function

# which prevents masking, or the default hiding of 
# a function with a redundant name from a package used next

date()

v <- R.Version()
print(paste0(v$language, " version ", v$major, ".",
 v$minor, " (", v$year, "-", v$month, "-", v$day, ")"))

library(lavaan)
library(semTools)

# get citation information
citation("lavaan", auto = TRUE)
citation("semTools", auto = TRUE)

# input the correlations in lower diagnonal form
duncanLower.cor <- '
1.000
 .640 1.000
 .586  .670 1.000
 .454  .566  .621 1.000
 .001  .038  .118  .091 1.000
 .214  .149  .135  .163  .025 1.000 '

# name the variables and convert to full correlation matrix
duncan.cor <- lavaan::getCov(duncanLower.cor, names = c("a1", "a2",
 "a3", "a4", "gender", "family"))

# display the correlations
duncan.cor

# add the standard deviations and convert to covariances
duncan.cov <- lavaan::cor2cov(duncan.cor, sds = c(1.002,.960,.912,.920,
 .504,.498))

# create mean vector
duncan.mean = c(2.271,2.560,2.694,2.965,.573,.446)

# display covariances and means
duncan.cov
duncan.mean

# maccallum-rmsea power
# exact fit test
# power at N = 321
semTools::findRMSEApower(0, .05, 9, 321, .05, 1)

# minimum N for power at least .90
semTools::findRMSEAsamplesize(0, .05, 9, .90, .05, 1)

# specify conditional growth model
predict.model <- '
# latent growth factors
 Intercept =~ 1*a1 + 1*a2 + 1*a3 + 1*a4
 Linear =~ 0*a1 + 1*a2 + 2*a3 + 3*a4
# covariates
 Intercept ~ gender + family
 Linear ~ gender + family '

# fit prediction model to data
predict <- lavaan::growth(predict.model, sample.cov = duncan.cov,
 sample.mean = duncan.mean, sample.nobs = 321)
lavaan::summary(predict, fit.measures = TRUE, rsquare = TRUE)

# observed indicator means
duncan.mean

# predicted means and covariances for observed variables
lavaan::fitted(predict)

# predicted means and correlation for factors
lavaan::lavInspect(predict, add.labels = TRUE, "mean.lv")
lavaan::lavInspect(predict, add.labels = TRUE, "cor.lv")

# residuals
lavaan::residuals(predict, type = "raw")
lavaan::residuals(predict, type = "standardized.mplus")
lavaan::residuals(predict, type = "cor.bollen")
