# principles and practice of sem, 5th ed.
# rex b. kline, guilford press, 2023

# chapter 21, table 21.2, analysis 1
# coefficients and effect sizes in basic
# basic growth models for change in neural risk 
# processing over ages 14–17 years

# to avoid the problem that some R packages share the same name 
# for different functions, all functions are specified next as 

# package::function

# which prevents masking, or the default hiding of 
# a function with a redundant name from a package used next

date()

v <- R.Version()
print(paste0(v$language, " version ", v$major, ".",
 v$minor, " (", v$year, "-", v$month, "-", v$day, ")"))

library(lavaan)
library(semTools)
 
# get citation information
citation("lavaan", auto = TRUE)
citation("semTools", auto = TRUE)

# read in summary statistics
kimspoonLower.cor <- '
1.00
 .35 1.00
 .28  .35 1.00
 .29  .40  .45 1.00
 .00  .07  .14  .04 1.00
 .00 -.09 -.12 -.10  .26 1.00 '
 
# name the variables and convert to full correlation matrix
kimspoon.cor <- lavaan::getCov(kimspoonLower.cor, names = c("R1", "R2",
 "R3", "R4", "abuse", "neglect"))
 
# display the correlations
kimspoon.cor

# add the standard deviations and convert to covariances
kimspoon.cov <- lavaan::cor2cov(kimspoon.cor, sds = c(.05,.77,.76,1.15,
 7.75,4.09))

# create mean vector
kimspoon.mean = c(.04,.61,.57,.83,7.17,3.03)
 
# display the covariances and means
kimspoon.cov
kimspoon.mean

# for all models, error variance for R1
# is fixed to equal zero

# lavaan function growth() automatically fixes
# intercepts of indicators to zero but specifies 
# latent growth factor means as free parameters 

# the direct tracing of the constant (delta-1) on
# the latent growth factors are means but are labeled
# as "intercepts" in lavaan output

# specify all models

# model 1
# no growth (intercept only)

noGrowth.model <- '
# specify intercept
# fix all loadings to 1.0
 Intercept =~ 1*R1 + 1*R2 + 1*R3 + 1*R4
# fix error variance for r1 to zero
 R1 ~~ 0*R1 '

# model 2
# latent basis growth model
# (curve fitting, level and shape)
# this model is retained

# maccallum-rmsea for model 2
# exact fit test
# power at N = 150
semTools::findRMSEApower(0, .05, 4, 150, .05, 1)

# minimum N for power at least .90
semTools::findRMSEAsamplesize(0, .05, 4, .90, .05, 1)

basis.model <- '
 Intercept =~ 1*R1 + 1*R2 + 1*R3 + 1*R4
# specify shape, first and last loadings fixed
 Shape =~ 0*R1 + R2 + R3 + 1*R4
 R1 ~~ 0*R1 '

# model 3
# linear growth model

linear.model <- '
 Intercept =~ 1*R1 + 1*R2 + 1*R3 + 1*R4 
# all loadings fixed to constants
 Linear =~ 0*R1 + 1*R2 + 2*R3 + 3*R4
 R1 ~~ 0*R1 '

# fit model 1 to data
noGrowth <- lavaan::growth(noGrowth.model, sample.cov = kimspoon.cov,
 sample.mean = kimspoon.mean, sample.nobs = 150)

# fit model 2 to data
basis <- lavaan::growth(basis.model, sample.cov = kimspoon.cov,
 sample.mean = kimspoon.mean, sample.nobs = 150)

# fit model 3 to data
linear <- lavaan::growth(linear.model, sample.cov = kimspoon.cov,
 sample.mean = kimspoon.mean, sample.nobs = 150)

# model chi-squares and chi-square difference tests
anova(noGrowth, basis)
anova(basis, linear)

# model 1 parameter estimates, global fit statistics,
# residuals
# very poor fit
lavaan::summary(noGrowth, fit.measures = TRUE, estimates = FALSE)
lavaan::fitted(noGrowth)
lavaan::residuals(noGrowth, type = "standardized")
lavaan::residuals(noGrowth, type = "cor.bollen")

# model 2 parameter estimates, global fit statistics,
# residuals
# retained model
lavaan::summary(basis, fit.measures = TRUE, rsquare = TRUE)
lavaan::standardizedSolution(basis)

# variance and standard error for Intercept are close to zero,
# so estimates are printed at 5-decimal accuracy, not 3 (default)
print(lavaan::parameterEstimates(basis), nd = 5)

# implied covariances and means for observed variables
lavaan::fitted(basis)
# implied means for latent growth factors
lavaan::lavInspect(basis, add.labels = TRUE, "mean.lv")

# residuals
lavaan::residuals(basis, type = "raw")
lavaan::residuals(basis, type = "standardized")
lavaan::residuals(basis, type = "cor.bollen")

# model 3 parameter estimates, global fit statistics,
# residuals
lavaan::summary(linear, fit.measures = TRUE)
lavaan::fitted(linear) 
lavaan::residuals(linear, type = "raw")
lavaan::residuals(linear, type = "standardized")
lavaan::residuals(linear, type = "cor.bollen")

