# principles and practice of sem, 5th ed.
# rex b. kline, guilford press, 2023

# chapter 22, table 22.1, analysis 1
# two-factor cfa model of divergent thinking 
# analyzed in separate chinese and american samples

# to avoid the problem that some R packages share the same name 
# for different functions, all functions are specified next as 

# package::function

# which prevents masking, or the default hiding of 
# a function with a redundant name from a package used next

date()

v <- R.Version()
print(paste0(v$language, " version ", v$major, ".",
 v$minor, " (", v$year, "-", v$month, "-", v$day, ")"))

library(lavaan)
 
# get citation information
citation("lavaan", auto = TRUE)

subtest.name <- c("LM1", "LM2", "LM3", "RW1", "RW2")

# chinese, n = 316, originality
chineseLower.cor <- '
1.000
 .715 1.000
 .631  .738 1.000
 .377  .408  .319 1.000
 .452  .495  .391  .445 1.000 '

# name the variables and convert to full correlation matrix
chinese.cor <- lavaan::getCov(chineseLower.cor, names = subtest.name)

# add the standard deviations and convert to covariances
chinese.sd <- c(1.532, 1.878, 2.012, 1.002, 1.055)
chinese.cov <- lavaan::cor2cov(chinese.cor, sds = chinese.sd)

# create mean vector
chinese.mean = c(1.24, 1.47, 1.86, .77, .89)

# american, n = 302, originality
americanLower.cor <- '
1.000
 .519 1.000
 .543  .639 1.000
 .280  .321  .317 1.000
 .242  .380  .300  .665 1.000 '

# name the variables and convert to full correlation matrix
american.cor <- lavaan::getCov(americanLower.cor, names = subtest.name)

# add the standard deviations and convert to covariances
american.sd <- c(1.094, 1.350, 1.494, 2.257, 1.541)
american.cov <- lavaan::cor2cov(american.cor, sds = american.sd)

# create mean vector
american.mean = c(.88, 1.35, 1.27, 1.71, 1.37)

# specify model for both single group analyses

single.model <- '
# turn off (NA) default reference variable scaling for
# first indicator
# label loadings and intercepts for both factors
 LineMeaning =~ NA*LM1 + a*LM1 + b*LM2 + c*LM3
 LM1 ~ d*1
 LM2 ~ e*1
 LM3 ~ f*1
 RealWorld =~ NA*RW1 + g*RW1 + h*RW2
 RW1 ~ i*1
 RW2 ~ j*1
# factor means are free parameters
 LineMeaning ~ 1
 RealWorld ~ 1 '

# specify effects coding constraints for each factor:
# average loading is 1, average intercept is zero

single.constraints <- '
# LineMeaning factor
 3 - a - b - c == 0
 0 - d - e - f == 0
# RealWorld factor
 2 - g - h == 0
 0 - i - j == 0'

# specify fit statistics for output
fit.stats.1 = c("chisq", "df", "pvalue", "cfi", "srmr")
fit.stats.2 = c("rmsea", "rmsea.ci.lower", "rmsea.ci.upper", "srmr")

# single group, chinese

chinese <- lavaan::cfa(single.model, sample.cov = chinese.cov, 
 sample.mean = chinese.mean, sample.nobs = 316,
 meanstructure = TRUE, constraints = single.constraints) 
lavaan::fitMeasures(chinese, fit.stats.1)
lavaan::fitMeasures(chinese, fit.stats.2)
lavaan::summary(chinese, standardized = TRUE, header = FALSE)
lavaan::fitted(chinese)
lavaan::residuals(chinese, type = "raw")
lavaan::residuals(chinese, type = "standardized")
lavaan::residuals(chinese, type = "cor.bollen")

# single group, american

american <- lavaan::cfa(single.model, sample.cov = american.cov, 
 sample.mean = american.mean, sample.nobs = 302,
 meanstructure = TRUE, constraints = single.constraints) 
lavaan::fitMeasures(american, fit.stats.1)
lavaan::fitMeasures(american, fit.stats.2)
lavaan::fitted (american)
lavaan::summary(american, standardized = TRUE, header = FALSE)
lavaan::residuals(american, type = "raw")
lavaan::residuals(american, type = "standardized")
lavaan::residuals(american, type = "cor.bollen")
