# principles and practice of sem, 5th ed.
# rex b. kline, guilford press, 2023

# chapter 8, table 8.1, analysis 3b
# local estimations of causal effects in the roth et al. (1989)
# parametric path model of illness through instruments (2sls)

# to avoid the problem that some R packages share the same name 
# for different functions, all functions are specified next as 

# package::function

# which prevents masking, or the default hiding of 
# a function with a redundant name from a package used next

date()

v <- R.Version()
print(paste0(v$language, " version ", v$major, ".",
 v$minor, " (", v$year, "-", v$month, "-", v$day, ")"))

library(systemfit)
library(semTools)
library(lavaan)

# packages semTools and lavaan are used to generate raw
# scores for analysis in package systemfit

# get citation information
citation("systemfit", auto = TRUE)
citation("semTools", auto = TRUE)
citation("lavaan", auto = TRUE)

# generate raw scores with descriptive statistics that 
# exactly match those from the summarey data in table 4.3
# reuses syntax from analysis 1 ("roth-generate-scores.r") to
# avoid reading in an external raw data file
# functions from semTools and lavaan

roth.cor <- matrix(c(1.00, -.03,  .39, -.05, -.08,
                     -.03, 1.00,  .07, -.23, -.16,
                      .39,  .07, 1.00, -.13, -.29,
                     -.05, -.23, -.13, 1.00,  .34,
                     -.08, -.16, -.29,  .34, 1.00), ncol = 5, nrow = 5)
roth.sd <- c(66.5, 38.0, 18.4, 33.5, 62.48)
roth.cov <- diag(roth.sd) %*% roth.cor %*% diag(roth.sd)
c = 372/373
roth.cov = c * roth.cov
roth.data <- semTools::kd(roth.cov, 373, type = "exact")
names(roth.data) <- c("exercise", "hardy", "fitness", "stress", "illness")
roth.data <- transform(roth.data, exercise = exercise + 40.9)
roth.data <- transform(roth.data, fitness = fitness + 67.1)
roth.data <- transform(roth.data, stress = stress + 24.0)
roth.data <- transform(roth.data, illness = illness + 71.67)

# direct effects

# exercise -> fitness, instr = hardy
ef_ih <- systemfit::systemfit(fitness ~ exercise, inst = ~ hardy,
 method = "2SLS", data = roth.data)
coef(summary(ef_ih))

# exercise -> fitness, instr = stress
ef_is <- systemfit::systemfit(fitness ~ exercise, inst = ~ stress, method = "2SLS",
 data = roth.data)
coef(summary(ef_is))

# hardy -> stress, instr = exercise
hs_ie <- systemfit::systemfit(stress ~ hardy, inst = ~ exercise, method = "2SLS",
 data = roth.data)
coef(summary(hs_ie))

# hardy -> stress, instr = fitness
hs_if <- systemfit::systemfit(stress ~ hardy, inst = ~ fitness, method = "2SLS",
 data = roth.data)
coef(summary(hs_if))

# compute conditional instruments listed next and add to data frame
# ef, exercise | fitness
# es, exercise | stress
# hf, hardy | fitness
# hs, hardy | stress

efr <- lm(exercise ~ fitness, data = roth.data)
esr <- lm(exercise ~ stress, data = roth.data)
hfr <- lm(hardy ~ fitness, data = roth.data)
hsr <- lm(hardy ~ stress, data = roth.data)

# add conditional instruments to data frame

roth.data$ef <- efr$resid
roth.data$es <- esr$resid
roth.data$hf <- hfr$resid
roth.data$hs <- hsr$resid

# verify independence of (ef, fitness), (es, stress),
# (hf, fitness), (hs, stress) within rounding error

options(scipen = 999)

cor(roth.data$ef, roth.data$fitness, method = "pearson")
cor(roth.data$es, roth.data$stress, method = "pearson")
cor(roth.data$hf, roth.data$fitness, method = "pearson")
cor(roth.data$hs, roth.data$stress, method = "pearson")

# fitness -> illness, instr = exercise | stress
fi_ies <- systemfit::systemfit(illness ~ fitness, inst = ~ es, method = "2SLS",
 data = roth.data)
coef(summary(fi_ies))

# fitness -> illness, instr = hardy | stress
fi_ihs <- systemfit::systemfit(illness ~ fitness, inst = ~ hs, method = "2SLS",
 data = roth.data)
coef(summary(fi_ihs))

# stress -> illness, instr = exercise | fitness
si_ief <- systemfit::systemfit(illness ~ stress, inst = ~ ef, method = "2SLS",
 data = roth.data)
coef(summary(si_ief))

# stress -> illness, instr = hardy | fitness
si_ihs <- systemfit::systemfit(illness ~ stress, inst = ~ hf, method = "2SLS",
 data = roth.data)
coef(summary(si_ihs))
